# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.template.loaders.app_directories import Loader as BaseLoader
from django.utils._os import safe_join
from domains.utils import get_template_name


try:
    # Django 1.7x or older
    from django.template.loaders.app_directories import app_template_dirs

except ImportError:
    # Django 1.8
    from django.template.utils import get_app_template_dirs
    app_template_dirs = get_app_template_dirs('templates')


class Loader(BaseLoader):
    is_usable = True

    def get_template_sources(self, template_name, template_dirs=None):
        """
        Looks in the saved request object from the middleware for
        directories and passes back the path. Doesn't verify that the
        path is valid, though.
        """
        if not template_dirs:
            template_dirs = app_template_dirs

        for template_dir in template_dirs:
            try:
                template_parts = get_template_name(template_dir, template_name)
                if not template_parts:
                    continue
                yield safe_join(*template_parts)
            except UnicodeDecodeError:
                # The template dir name was a bytestring
                # that wasn't valid UTF-8.
                raise
            except ValueError:
                # The joined path was located outside of template_dir.
                pass



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from domains.utils import get_hooks, setup_hook

for hook in get_hooks():
    setup_hook(hook)



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.conf import settings
from django.template.loaders.filesystem import Loader as BaseLoader
from django.utils._os import safe_join
from domains.utils import get_template_name


class Loader(BaseLoader):
    is_usable = True

    def get_template_sources(self, template_name, template_dirs=None):
        """
        Returns the absolute paths to "template_name", when appended to each
        directory in "template_dirs". Any paths that don't lie inside one of
        the template dirs are excluded from the result set, for
        security reasons.
        """
        if not template_dirs:
            template_dirs = settings.TEMPLATE_DIRS

        for template_dir in template_dirs:
            try:
                template_parts = get_template_name(template_dir, template_name)
                if not template_parts:
                    continue
                yield safe_join(*template_parts)

            except UnicodeDecodeError:
                # The template dir name was a bytestring that
                # wasn't valid UTF-8.
                raise
            except ValueError:
                # The joined path was located outside of this particular
                # template_dir (it might be inside another one, so this isn't
                # fatal).
                pass



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from domains.utils import set_thread_variable, get_installed_hooks
import warnings


class RequestMiddleware(object):
    """
    Stores the request object in the local thread
    """
    def process_request(self, request):
        set_thread_variable('request', request)


class DomainSettingsPatchMiddleware(object):
    def process_request(self, request):
        for attr, hook in get_installed_hooks().items():
            hook.apply(request)


class DynamicSiteMiddleware(DomainSettingsPatchMiddleware):
    """
    Define current Django Site for requested hostname
    """
    def __init__(self):
        super(DynamicSiteMiddleware, self).__init__()
        warnings.warn(
            "domains.middleware.DynamicSiteMiddleware is deprecated. "
            "Please use domains.middleware.DomainSettingsPatchMiddleware "
            "instead"
        )



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.conf import settings
from django.contrib.sites.models import Site
from django.db.models.loading import get_model
from domains.hooks.base import IntHookBase


HOST_CACHE = {}


def get_site_model():
    """
    Returns Site model
    """
    if 'django.contrib.sites' in settings.INSTALLED_APPS:
        return Site
    if getattr(settings, 'DOMAINS_SITE_MODEL', None):
        return get_model(settings.DOMAIN_SITE_MODEL)
    return None


def get_site_by_host(hostname, field=None):
    site_model = get_site_model()
    field = field or getattr(settings, 'DOMAINS_SITE_FIELD', 'domain')
    try:
        return site_model.objects.get(**{field: hostname})
    except site_model.DoesNotExist:
        pass


class SiteIDHook(IntHookBase):
    """
    Dynamic SITE_ID attribute class
    """
    attribute = 'SITE_ID'
    default_value = 1

    def __init__(self, *args, **kwargs):
        super(IntHookBase, self).__init__(*args, **kwargs)
        self.sites_support = get_site_model() is not None

    def cache(self, host, value):
        HOST_CACHE[host] = value

    def set_and_cache(self, host, site_id):
        self.set(site_id)
        self.cache(host, site_id)

    def apply(self, request):
        if not self.sites_support:
            return
        host = request.get_host()
        shost = host.rsplit(':', 1)[0]  # just host, no port

        if host in HOST_CACHE:
            self.set(HOST_CACHE[host])
            return
        site = get_site_by_host(host)

        if site:
            self.set_and_cache(host, site.pk)
            return

        if shost != host:  # get by hostname without port
            site = get_site_by_host(shost)
            if site:
                self.set_and_cache(host, site.pk)
                return
        site = get_site_by_host(settings.SITE_ID, field='pk')
        if site:
            self.cache(host, site.pk)
            return

        try:  # misconfigured settings?
            site = get_site_model().objects.all()[0]
            self.set_and_cache(host, site.pk)
            return
        except IndexError:  # no sites in db
            pass



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from domains.compat import import_by_path, local


# Threaded variable namespace
_thread_locals = local()
installed_hooks = {}


def get_hooks():
    return getattr(
        settings, 'DOMAINS_HOOKS',
        (
            'domains.hooks.site_id.SiteIDHook',
        ))


def setup_hook(hook):
    from domains.hooks.base import HookBase

    hook_cls = import_by_path(hook)
    hook_instance = hook_cls()

    if not isinstance(hook_instance, HookBase):
        raise ImproperlyConfigured(
            '%s is not a HookBase instance' % hook
        )

    installed_hooks[hook_instance.attribute] = hook_instance
    setattr(settings, hook_instance.attribute, hook_instance)


def get_installed_hooks():
    return installed_hooks


def set_thread_variable(key, var):
    """
    Sets the threaded variable
    """
    setattr(_thread_locals, key, var)


def get_thread_variable(key, default=None):
    """
    Gets the threaded variable
    """
    return getattr(_thread_locals, key, default)


def get_hostname():
    """
    Returns the current domain name in lower case and without port number
    """
    request = get_request()
    if request is None:
        return
    return request.get_host().split(':')[0].lower()


def get_template_name(template_dir, template_name):
    """
    Generates parts of template name
    """
    custom_function = getattr(settings, 'DOMAINS_TEMPLATE_NAME_FUNCTION', None)

    if custom_function is not None:
        return import_by_path(custom_function)(template_dir, template_name)
    hostname = get_hostname()
    return template_dir, hostname, template_name if hostname else None


def get_request():
    """
    Returns request object from current thread
    """
    return get_thread_variable('request')



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.conf import settings


HOOKS = getattr(settings, 'DOMAINS_HOOKS',
                (
                    'domains.hooks.SiteIDHook',
                ))



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.core.exceptions import ImproperlyConfigured
from domains.utils import _thread_locals
from domains.compat import text_type


class HookBase(object):
    attribute = None
    default_value = ''

    def __init__(self, *args, **kwargs):
        self._args = args
        self._kwargs = kwargs
        if self.attribute is None:
            raise ImproperlyConfigured(
                "Please specify `attribute` into  your hook class"
            )

    def __repr__(self):
        v = self.get()
        return text_type(v)

    def __hash__(self):
        return self.get()

    def coerce(self, v):
        return v

    def get(self):
        v = getattr(_thread_locals, self.attribute, self.default_value)
        return self.coerce(v)

    def set(self, value):
        setattr(_thread_locals, self.attribute, self.coerce(value))

    def value(self, request):
        raise NotImplementedError()

    def apply(self, request):
        self.set(self.value(request))


class IntHookBase(HookBase, int):
    default_value = 0

    def coerce(self, v):
        return int(v)

    def __int__(self):
        try:
            return self.get()
        except AttributeError:
            self.set(self.default_value)
            return self.get()


class StrHookBase(HookBase, text_type):
    default_value = ''

    def coerce(self, v):
        return text_type(v)

    def __str__(self):
        try:
            return self.get()
        except AttributeError:
            self.set(self.default_value)
            return self.get()

    __unicode__ = __str__


class IterMixin(object):
    def __iter__(self):
        return iter(self.get())


class TupleHookBase(HookBase, IterMixin, tuple):
    default_value = ()
    base_type = tuple


class ListHookBase(HookBase, IterMixin, list):
    default_value = []
    base_type = list


class DictHookBase(HookBase, IterMixin, object):
    default_value = {}
    base_type = dict

    def __iter__(self):
        return iter(self.get().items())

    def __getattr__(self, name):
        return self.get()[name]



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, find_packages
import os


ROOT_PACKAGE = 'django-domains'
DIR = os.path.dirname(__file__)
VERSION = '0.5.1'


def long_description():
    """
    Returns package long description from README
    """
    def read(what):
        with open(os.path.join(DIR, '%s.rst' % what)) as fp:
            return fp.read()

    return "{README}\n\n{CHANGELOG}".format(README=read('README'),
                                            CHANGELOG=read('CHANGELOG'))


def version():
    """
    Returns package version for package building
    """
    return VERSION


if __name__ == '__main__':
    setup(name=ROOT_PACKAGE,
          description=('An application that allows you to run many '
                       'different sites on one Django instance'),
          author='Mikhail Porokhovnichenko',
          author_email='marazmiki@gmail.com',
          version=version(),
          long_description=long_description(),
          packages=find_packages(),
          include_package_data=True,
          test_suite='tests.main',
          zip_safe=False,
          classifiers=[
              'Environment :: Web Environment',
              'Programming Language :: Python',
              'Framework :: Django'
          ])



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.utils.importlib import import_module


try:
    from django.utils import six
    text_type = six.text_type
except ImportError:
    text_type = str

try:
    from threading import local    # NOQA
except ImportError:
    from django.utils._threading_local import local    # NOQA


try:
    from django.utils.module_loading import import_by_path

except ImportError:
    from django.core.exceptions import ImproperlyConfigured

    def import_by_path(dotted_path, error_prefix=''):
        """
        Import a dotted module path and return the attribute/class
        designated by the last name in the path. Raise ImproperlyConfigured
        if something goes wrong.
        """
        try:
            module_path, class_name = dotted_path.rsplit('.', 1)
        except ValueError:
            raise ImproperlyConfigured(
                "%s%s doesn't look like a module path" % (
                    error_prefix, dotted_path
                )
            )
        try:
            module = import_module(module_path)
        except ImportError as e:
            import sys
            msg = '%sError importing module %s: "%s"' % (
                error_prefix, module_path, e)
            try:
                from django.utils import six
                six.reraise(ImproperlyConfigured, ImproperlyConfigured(msg),
                            sys.exc_info()[2])
            except ImportError:
                raise ImproperlyConfigured(msg)
        try:
            attr = getattr(module, class_name)
        except AttributeError:
            raise ImproperlyConfigured(
                '%sModule "%s" does not define a "%s" attribute/class' % (
                    error_prefix, module_path, class_name
                )
            )
        return attr



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django import test, get_version
from django.core.exceptions import ImproperlyConfigured
from django.conf import settings
from django.conf.urls import url
from django.contrib.sites.models import Site
from django.shortcuts import render
from domains import setup_hook
from domains.compat import text_type
from domains.hooks.base import (StrHookBase, TupleHookBase, ListHookBase,
                                DictHookBase, IntHookBase)


urlpatterns = [
    url('^$', lambda request: render(request, 'test_index.html')),
]


def hook_str(h):
    return h


def hook_int(h):
    return len(h)


def hook_dict(h):
    return {1: h + '_1', 2: h + '_2'}


def hook_list(h):
    return [h, h, h]


def hook_tuple(h):
    return (h, h, h)


def test_function(template_dir, template_name):
    """
    Generates parts of template name
    """
    return template_dir, 'custom_' + template_name


class TestHookBase(object):
    hook_function = hook_str

    def value(self, request):
        return globals()[self.hook_function](request.get_host())


class TestIntHook(TestHookBase, IntHookBase):
    """
    Test cases for integer variables hook
    """
    attribute = 'DOMAINS_TEST_ATTRIBUTE_INT'
    hook_function = 'hook_int'


class TestStrHook(TestHookBase, StrHookBase):
    """
    Test cases for string variables hook
    """
    attribute = 'DOMAINS_TEST_ATTRIBUTE_STR'
    hook_function = 'hook_str'


class TestDictHook(TestHookBase, DictHookBase):
    """
    Test cases for dictionary variables hook
    """
    attribute = 'DOMAINS_TEST_ATTRIBUTE_DICT'
    hook_function = 'hook_dict'


class TestListHook(TestHookBase, ListHookBase):
    """
    Test cases for list variables hook
    """
    attribute = 'DOMAINS_TEST_ATTRIBUTE_LIST'
    hook_function = 'hook_list'


class TestTupleHook(TestHookBase, TupleHookBase):
    """
    Test cases for tuple variables hook
    """
    attribute = 'DOMAINS_TEST_ATTRIBUTE_TUPLE'
    hook_function = 'hook_tuple'


class EnvironmentTest(test.TestCase):
    """
    Tests the environment
    """
    def test_middleware(self):
        self.assertIn(
            'domains.middleware.RequestMiddleware',
            settings.MIDDLEWARE_CLASSES
        )


class TestBase(test.TestCase):
    """
    Common things
    """
    urls = 'domains.tests'

    def setUp(self):
        """
        Runs before each unit test
        """
        self.client = test.Client()


class TemplateLoadersTest(TestBase):
    """
    Test suite for dynamic template directories
    """

    def test_foo_dot_com(self):
        """
        Tests for first domain
        """
        resp = self.client.get('/', HTTP_HOST='test.foo.com')
        self.assertContains(resp, text='test.foo.com')
        self.assertNotContains(resp, text='test.bar.com')

    def test_bar_dot_com(self):
        """
        Tests for second domain
        """
        resp = self.client.get('/', HTTP_HOST='test.bar.com')
        self.assertContains(resp, 'test.bar.com')
        self.assertNotContains(resp, 'test.foo.com')

    def test_another_domain(self):
        """
        Tests if templates with domain name not found
        """
        resp = self.client.get('/', HTTP_HOST='github.com')
        self.assertNotContains(resp, 'test.bar.com')
        self.assertNotContains(resp, 'test.foo.com')
        self.assertContains(resp, 'DEFAULT')

    def test_complex(self):
        """
        Complex
        """
        tests = (
            ('test.foo.com', 'test.foo.com'),
            ('microsoft.com', 'DEFAULT'),
            ('test.foo.com', 'test.foo.com'),
            ('test.bar.com', 'test.bar.com'),
            ('google.com', 'DEFAULT'),
            ('test.bar.com', 'test.bar.com'),
            ('test.foo.com', 'test.foo.com'),
            ('yahoo.com', 'DEFAULT'),
        )

        for domain, content in tests:
            resp = self.client.get('/', HTTP_HOST=domain)
            self.assertContains(resp, content)

    def test_custom_function(self):
        """
        Custom function
        """
        func = 'domains.tests.test_function'
        with self.settings(DOMAINS_TEMPLATE_NAME_FUNCTION=func):
            resp = self.client.get('/', HTTP_HOST='microsoft.com')
            self.assertContains(resp, 'CUSTOM')

    def test_hook_str(self):
        domain = 'test.foo.com'
        self.client.get('/', HTTP_HOST=domain)
        self.assertEquals(hook_str(domain),
                          text_type(settings.DOMAINS_TEST_ATTRIBUTE_STR))

    def test_hook_int(self):
        domain = 'test.foo.com'
        self.client.get('/', HTTP_HOST=domain)
        self.assertEquals(hook_int(domain),
                          int(settings.DOMAINS_TEST_ATTRIBUTE_INT))

    def test_hook_tuple(self):
        domain = 'test.foo.com'
        self.client.get('/', HTTP_HOST=domain)
        self.assertEquals(hook_tuple(domain),
                          tuple(settings.DOMAINS_TEST_ATTRIBUTE_TUPLE))

    def test_hook_list(self):
        domain = 'test.foo.com'
        self.client.get('/', HTTP_HOST=domain)
        self.assertEquals(hook_list(domain),
                          list(settings.DOMAINS_TEST_ATTRIBUTE_LIST))

    def test_hook_dict(self):
        domain = 'test.foo.com'
        self.client.get('/', HTTP_HOST=domain)
        self.assertDictEqual(hook_dict(domain),
                             dict(settings.DOMAINS_TEST_ATTRIBUTE_DICT))


class SiteIdTest(TestBase):
    """
    Test dynamic
    """
    def setUp(self):
        super(SiteIdTest, self).setUp()
        self.sites = {
            1: Site.objects.create(domain='test.foo.com', name='Foo'),
            2: Site.objects.create(domain='test.bar.com', name='Bar'),
            3: Site.objects.create(domain='github.com', name='Git'),
        }

    def test_1(self):
        for _ in range(0, 3):
            for pk, site in self.sites.items():
                resp = self.client.get('/', HTTP_HOST=site.domain)

                if get_version() >= '1.8':
                    curr = Site.objects.get_current(resp.context['request'])
                else:
                    curr = Site.objects.get_current()

                self.assertEquals(curr.domain, site.domain)
                self.assertEquals(curr.pk, site.pk)


class TestCore(test.TestCase):
    def test_improperly_hook_class(self):
        self.assertRaises(ImproperlyConfigured,
                          lambda: setup_hook('hook.that.DoesNotExist'))
